@@ -236,7 +236,7 @@ pub trait Rpc {
     /// pub/sub and send broadcast messages)
     fn client(&self) -> Arc<Mutex<(dyn AsyncClient + 'static)>>;
     async fn notify(
-        &mut self,
+        &self,
         target: &str,
         data: Cow<'async_trait>,
         qos: QoS,
@@ -247,20 +247,22 @@ pub trait Rpc {
         target: &str,
         method: &str,
         params: Cow<'async_trait>,
+        qos: QoS,
     ) -> Result<OpConfirm, Error>;
     /// Call the method and get the response
     async fn call(
-        &mut self,
+        &self,
         target: &str,
         method: &str,
         params: Cow<'async_trait>,
+        qos: QoS,
     ) -> Result<RpcEvent, RpcError>;
     fn is_connected(&self) -> bool;
 }
 
 #[allow(clippy::module_name_repetitions)]
 pub struct RpcClient {
-    call_id: u32,
+    call_id: atomic::AtomicU32,
     timeout: Option<Duration>,
     client: Arc<Mutex<dyn AsyncClient>>,
     processor_fut: Arc<std::sync::Mutex<JoinHandle<()>>>,
@@ -305,6 +307,11 @@ async fn processor<C, H>(
                         };
                         let h = handlers.clone();
                         tokio::spawn(async move {
+                            let qos = if event.frame().is_realtime() {
+                                QoS::RealtimeProcessed
+                            } else {
+                                QoS::Processed
+                            };
                             let res = h.handle_call(event).await;
                             if let Some((target, cl)) = ev {
                                 macro_rules! send_reply {
@@ -312,21 +319,11 @@ async fn processor<C, H>(
                                         let mut client = cl.lock().await;
                                         if let Some(result) = $result {
                                             client
-                                                .zc_send(
-                                                    &target,
-                                                    $payload,
-                                                    result.into(),
-                                                    QoS::Processed,
-                                                )
+                                                .zc_send(&target, $payload, result.into(), qos)
                                                 .await
                                         } else {
                                             client
-                                                .zc_send(
-                                                    &target,
-                                                    $payload,
-                                                    (&[][..]).into(),
-                                                    QoS::Processed,
-                                                )
+                                                .zc_send(&target, $payload, (&[][..]).into(), qos)
                                                 .await
                                         }
                                     }};
@@ -426,7 +423,7 @@ impl RpcClient {
             })
         });
         Self {
-            call_id: 0,
+            call_id: atomic::AtomicU32::new(1),
             timeout,
             client,
             processor_fut,
@@ -436,12 +433,14 @@ impl RpcClient {
         }
     }
     #[inline]
-    fn increase_call_id(&mut self) {
-        if self.call_id == u32::MAX {
-            self.call_id = 1;
-        } else {
-            self.call_id += 1;
-        }
+    fn generate_call_id(&self) -> u32 {
+        self.call_id
+            .fetch_update(
+                atomic::Ordering::SeqCst,
+                atomic::Ordering::SeqCst,
+                |call_id| Some(if call_id == u32::MAX { 1 } else { call_id + 1 }),
+            )
+            .unwrap()
     }
 }
 
@@ -453,7 +452,7 @@ impl Rpc for RpcClient {
     }
     #[inline]
     async fn notify(
-        &mut self,
+        &self,
         target: &str,
         data: Cow<'async_trait>,
         qos: QoS,
@@ -469,25 +468,26 @@ impl Rpc for RpcClient {
         target: &str,
         method: &str,
         params: Cow<'async_trait>,
+        qos: QoS,
     ) -> Result<OpConfirm, Error> {
         let payload = prepare_call_payload(method, &[0, 0, 0, 0]);
         self.client
             .lock()
             .await
-            .zc_send(target, payload.into(), params, QoS::Processed)
+            .zc_send(target, payload.into(), params, qos)
             .await
     }
     /// # Panics
     ///
     /// Will panic on poisoned mutex
     async fn call(
-        &mut self,
+        &self,
         target: &str,
         method: &str,
         params: Cow<'async_trait>,
+        qos: QoS,
     ) -> Result<RpcEvent, RpcError> {
-        self.increase_call_id();
-        let call_id = self.call_id;
+        let call_id = self.generate_call_id();
         let payload = prepare_call_payload(method, &call_id.to_le_bytes());
         let (tx, rx) = oneshot::channel();
         self.calls.lock().unwrap().insert(call_id, tx);
@@ -502,29 +502,19 @@ impl Rpc for RpcClient {
                 }
             };
         }
-        {
+        let opc = {
             let mut client = self.client.lock().await;
-            let fut = client.zc_send(target, payload.into(), params, QoS::Processed);
+            let fut = client.zc_send(target, payload.into(), params, qos);
             if let Some(timeout) = self.timeout {
-                unwrap_or_cancel!(unwrap_or_cancel!(
-                    unwrap_or_cancel!(unwrap_or_cancel!(tokio::time::timeout(timeout, fut).await))
-                        .unwrap()
-                        .await
-                ));
+                unwrap_or_cancel!(unwrap_or_cancel!(tokio::time::timeout(timeout, fut).await))
             } else {
-                unwrap_or_cancel!(unwrap_or_cancel!(
-                    unwrap_or_cancel!(fut.await).unwrap().await
-                ));
+                unwrap_or_cancel!(fut.await)
             }
-        }
-        let result = if let Some(timeout) = self.timeout {
-            tokio::time::timeout(timeout, rx)
-                .await
-                .map_err(Into::<Error>::into)?
-                .map_err(Into::<Error>::into)?
-        } else {
-            rx.await.map_err(Into::<Error>::into)?
         };
+        if let Some(c) = opc {
+            unwrap_or_cancel!(unwrap_or_cancel!(c.await));
+        }
+        let result = rx.await.map_err(Into::<Error>::into)?;
         if let Ok(e) = TryInto::<RpcError>::try_into(&result) {
             Err(e)
         } else {
@@ -570,33 +560,45 @@ impl RpcError {
         Self { code, data }
     }
     #[inline]
-    pub fn method() -> Self {
+    pub fn method(err: Option<Vec<u8>>) -> Self {
         Self {
             code: RPC_ERROR_CODE_METHOD_NOT_FOUND,
-            data: None,
+            data: err,
         }
     }
     #[inline]
-    pub fn params() -> Self {
+    pub fn params(err: Option<Vec<u8>>) -> Self {
         Self {
             code: RPC_ERROR_CODE_INVALID_METHOD_PARAMS,
-            data: None,
+            data: err,
         }
     }
     #[inline]
-    pub fn parse() -> Self {
+    pub fn parse(err: Option<Vec<u8>>) -> Self {
         Self {
             code: RPC_ERROR_CODE_PARSE,
-            data: None,
+            data: err,
         }
     }
     #[inline]
-    pub fn invalid() -> Self {
+    pub fn invalid(err: Option<Vec<u8>>) -> Self {
         Self {
             code: RPC_ERROR_CODE_INVALID_REQUEST,
-            data: None,
+            data: err,
         }
     }
+    #[inline]
+    pub fn internal(err: Option<Vec<u8>>) -> Self {
+        Self {
+            code: RPC_ERROR_CODE_INTERNAL,
+            data: err,
+        }
+    }
+    /// Converts displayable to Vec<u8>
+    #[inline]
+    pub fn convert_data(v: impl fmt::Display) -> Vec<u8> {
+        v.to_string().as_bytes().to_vec()
+    }
 }
 
 impl From<Error> for RpcError {
